

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

The #PangoEngineScriptInfo structure contains
information about how the shaper covers a particular script.
-}

module GI.Pango.Structs.EngineScriptInfo
    ( 

-- * Exported types
    EngineScriptInfo(..)                    ,
    newZeroEngineScriptInfo                 ,
    noEngineScriptInfo                      ,


 -- * Properties
-- ** Langs
    engineScriptInfoReadLangs               ,


-- ** Script
    engineScriptInfoReadScript              ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Pango.Types
import GI.Pango.Callbacks

newtype EngineScriptInfo = EngineScriptInfo (ForeignPtr EngineScriptInfo)
-- | Construct a `EngineScriptInfo` struct initialized to zero.
newZeroEngineScriptInfo :: MonadIO m => m EngineScriptInfo
newZeroEngineScriptInfo = liftIO $ callocBytes 16 >>= wrapPtr EngineScriptInfo

noEngineScriptInfo :: Maybe EngineScriptInfo
noEngineScriptInfo = Nothing

engineScriptInfoReadScript :: EngineScriptInfo -> IO Script
engineScriptInfoReadScript s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO CUInt
    let val' = (toEnum . fromIntegral) val
    return val'

engineScriptInfoReadLangs :: EngineScriptInfo -> IO T.Text
engineScriptInfoReadLangs s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CString
    val' <- cstringToText val
    return val'

type family ResolveEngineScriptInfoMethod (t :: Symbol) (o :: *) :: * where
    ResolveEngineScriptInfoMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveEngineScriptInfoMethod t EngineScriptInfo, MethodInfo info EngineScriptInfo p) => IsLabelProxy t (EngineScriptInfo -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveEngineScriptInfoMethod t EngineScriptInfo, MethodInfo info EngineScriptInfo p) => IsLabel t (EngineScriptInfo -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


