

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

The #PangoGlyphInfo structure represents a single glyph together with
positioning information and visual attributes.
It contains the following fields.
-}

module GI.Pango.Structs.GlyphInfo
    ( 

-- * Exported types
    GlyphInfo(..)                           ,
    newZeroGlyphInfo                        ,
    noGlyphInfo                             ,


 -- * Properties
-- ** Attr
    glyphInfoReadAttr                       ,


-- ** Geometry
    glyphInfoReadGeometry                   ,


-- ** Glyph
    glyphInfoReadGlyph                      ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Pango.Types
import GI.Pango.Callbacks

newtype GlyphInfo = GlyphInfo (ForeignPtr GlyphInfo)
-- | Construct a `GlyphInfo` struct initialized to zero.
newZeroGlyphInfo :: MonadIO m => m GlyphInfo
newZeroGlyphInfo = liftIO $ callocBytes 20 >>= wrapPtr GlyphInfo

noGlyphInfo :: Maybe GlyphInfo
noGlyphInfo = Nothing

glyphInfoReadGlyph :: GlyphInfo -> IO Word32
glyphInfoReadGlyph s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Word32
    return val

glyphInfoReadGeometry :: GlyphInfo -> IO GlyphGeometry
glyphInfoReadGeometry s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 4) :: IO (Ptr GlyphGeometry)
    val' <- (newPtr 12 GlyphGeometry) val
    return val'

glyphInfoReadAttr :: GlyphInfo -> IO GlyphVisAttr
glyphInfoReadAttr s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO (Ptr GlyphVisAttr)
    val' <- (newPtr 4 GlyphVisAttr) val
    return val'

type family ResolveGlyphInfoMethod (t :: Symbol) (o :: *) :: * where
    ResolveGlyphInfoMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveGlyphInfoMethod t GlyphInfo, MethodInfo info GlyphInfo p) => IsLabelProxy t (GlyphInfo -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveGlyphInfoMethod t GlyphInfo, MethodInfo info GlyphInfo p) => IsLabel t (GlyphInfo -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


